import json


VERSION = "5.199"


def get_event_params(event: dict, event_type: str) -> dict:
    text = None
    user_id = None
    peer_id = None
    post_id = None
    owner_id = None
    object_id = None
    reply = {}
    action = {}
    payload = {}

    try:
        if event_type == "message_new":
            text = event["object"]["message"]["text"]
            peer_id = event["object"]["message"]["peer_id"]
            user_id = event["object"]["message"]["from_id"]
            object_id = event["object"]["message"]["conversation_message_id"]

            if "reply_message" in event["object"]["message"]:
                reply = {
                    "text": event["object"]["message"]["reply_message"]["text"],
                    "peer_id": event["object"]["message"]["reply_message"]["peer_id"],
                    "user_id": event["object"]["message"]["reply_message"]["from_id"],
                    "object_id": event["object"]["message"]["reply_message"]["conversation_message_id"]
                }

            if "action" in event["object"]["message"]:
                action = {
                    "type": event["object"]["message"]["action"]["type"],
                    "object_id": event["object"]["message"]["action"]["conversation_message_id"],
                    "member_id": event["object"]["message"]["action"]["member_id"],
                    "text": event["object"]["message"]["action"]["message"]
                }

            if "payload" in event["object"]["message"]:
                payload = json.loads(event["object"]["message"]["payload"])

        elif event_type == "message_edit":
            text = event["object"]["text"]
            peer_id = event["object"]["peer_id"]
            user_id = event["object"]["from_id"]
            object_id = event["object"]["conversation_message_id"]

            if "reply_message" in event["object"]:
                reply = {
                    "text": event["object"]["reply_message"]["text"],
                    "peer_id": event["object"]["reply_message"]["peer_id"],
                    "user_id": event["object"]["reply_message"]["from_id"],
                    "object_id": event["object"]["reply_message"]["conversation_message_id"]
                }

        elif event_type in ("wall_reply_new", "wall_reply_edit"):
            text = event["object"]["text"]
            owner_id = event["object"]["owner_id"]
            user_id = event["object"]["from_id"]
            post_id = event["object"]["post_id"]

        elif event_type == "wall_post_new":
            text = event["object"]["text"]
            owner_id = event["object"]["owner_id"]
            user_id = event["object"]["from_id"]
            post_id = event["object"]["id"]

        elif event_type in ("board_post_new", "board_post_edit"):
            text = event["object"]["text"]
            owner_id = event["group_id"]
            user_id = event["object"]["from_id"]
            post_id = event["object"]["topic_id"]

    except KeyError:
        pass

    return {
        "type": event_type,
        "text": text,
        "user_id": user_id,
        "peer_id": peer_id,
        "post_id": post_id,
        "owner_id": owner_id,
        "object_id": object_id,
        "reply": reply,
        "action": action,
        "payload": payload
    }



from typing import List, Tuple, Any
from copy import deepcopy
from .handlers import Handler


class Reject:
    pass


class Reset:
    pass


class BoundChain:
    def __init__(self, primal_chain, handlers: List[Handler], user_id: int):
        self.primal_chain = primal_chain
        self.handlers = handlers
        self.user_id = user_id
        self.data = None

    def is_trigger(self, event_type: str, user_id: int) -> bool:
        return event_type == self.handlers[0].event_type and user_id == self.user_id

    async def new_event(self, bot, event: dict, event_params: dict) -> Tuple:
        handler = self.handlers[0]
        if handler.is_trigger(event_params):
            data = await handler.new_event(bot, event, event_params, self.data)
            if isinstance(data, Reject):
                return False, self
        else:
            return False, self

        if not isinstance(data, Reset):
            self.handlers.pop(0)
            self.data = data

        if len(self.handlers) < 1:
            return False, self

        return True, self


class Chain:
    def __init__(self):
        self.handlers = []

    def add_handler(self, handler: Handler) -> Handler:
        self.handlers.append(handler)
        return handler

    def is_trigger(self, event_params: dict) -> bool:
        return self.handlers[0].is_trigger(event_params)

    def bind_chain(self, user_id: int) -> BoundChain:
        return BoundChain(self, deepcopy(self.handlers), user_id)



import asyncio
import aiohttp
import traceback
import sys

from . import handlers
from .chain import Chain
from .core import VERSION, get_event_params


class Handler:
    on = handlers.Handlers


class Bot:
    def __init__(self, token: str, group_id: int):
        self.session = aiohttp.ClientSession()
        self.token = token
        self.group_id = group_id
        self.config = {}
        self.handlers = []
        self.users_in_chain = []
        self.chains = []
        self.bound_chains = []

    def handle(self, handler: handlers.Handler) -> handlers.Handler:
        self.handlers.append(handler)
        return handler

    def add_chain(self, chain: Chain) -> Chain:
        self.chains.append(chain)
        return chain

    async def run_polling(self) -> None:
        async with self.session.get("https://api.vk.com/method/groups.getLongPollServer",
                                    params=f"group_id={self.group_id}&access_token={self.token}&v={VERSION}") as response:
            config = await response.json()
            print(config)
            self.config = config["response"]
            self.config["server"] = self.config["server"].replace("https://", "")

            async with self.session.get("https://api.vk.com/method/groups.getLongPollSettings",
                                        params=f"group_id={self.group_id}&access_token={self.token}&v={VERSION}") as response:
                print(await response.json())

            while True:
                try:
                    async with self.session.get(f"https://%s?act=a_check&key=%s&ts=%s&wait=60&mode=2&version={VERSION}" % (
                               self.config["server"], self.config["key"], self.config["ts"])) as response:
                        event = await response.json()
                        updates = event["updates"]
                        if len(updates) == 0:
                            continue
                        print(updates)

                        self.config["ts"] = event["ts"]

                        asyncio.create_task(self.send_event(updates[0]))

                except:
                    sys.stderr.write(f"\n\n{traceback.format_exc()}\n\n")

    async def send_event(self, event: dict) -> None:
        event_type = event["type"]
        event_params = get_event_params(event, event_type)

        if event_params["user_id"] not in self.users_in_chain:
            for chain in self.chains:
                if chain.is_trigger(event_params):
                    self.bound_chains.append(chain.bind_chain(event_params["user_id"]))
                    self.users_in_chain.append(event_params["user_id"])
                    break

        active_chains = [chain for chain in self.bound_chains if chain.is_trigger(event_type, event_params["user_id"])]
        tasks = [asyncio.create_task(chain.new_event(self, event, event_params))
                 for chain in active_chains]

        for future in asyncio.as_completed(tasks):
            result = await future
            if not result[0]:
                self.bound_chains.remove(result[1])
                self.users_in_chain.remove(event_params["user_id"])

        active_handlers = [handler for handler in self.handlers if handler.is_trigger(event_params)]
        tasks = [asyncio.create_task(handler.new_event(self, event, event_params))
                 for handler in active_handlers]
        await asyncio.gather(*tasks)

    async def execute(self, method: str, **params) -> dict:
        url = f"https://api.vk.com/method/{method}"
        query = ""
        for key, value in params.items():
            if value is not None:
                query += f"{key}={value}&"
        query += f"access_token={self.token}&v={VERSION}"

        async with self.session.post(url, params=query) as response:
            return await response.json()


def run_polling(bot: Bot) -> None:
    loop = asyncio.get_event_loop()
    loop.run_until_complete(bot.run_polling())



from setuptools import setup, find_packages


with open("README.md", "r", encoding="UTF-8") as file:
    readme = file.read()


requirements = ["asyncio>=3", "aiohttp>=3", "aiofiles>=22", "aiosqlite>=0.20"]


setup(
    name="async_VKsher",
    version="2.2.1",
    author="Kulenov Islam",
    author_email="kit.werr34@gmail.com",
    description="asyncVK is asynchronous library for creating a bot in VK",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/Ekventor/asyncVK",
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ],
    keywords="vk vkontakte вк вконтакте бот bot"
)



import json
from typing import Union, Iterable
import json

from .keyboard import Keyboard


class Message:
    def __init__(self, bot, peer_id: int, message_id: int):
        self.bot = bot
        self.peer_id = peer_id
        self.message_id = message_id

    async def edit(self, text: str = None, attachment: str = None, keyboard: Union[str, Keyboard] = None) -> None:
        await self.bot.execute("messages.edit", peer_id=self.peer_id, conversation_message_id=self.message_id,
                               message=text, attachment=attachment, keyboard=keyboard, keep_forward_messages=1,
                               keep_snippets=1)

    async def delete(self) -> None:
        await self.bot.execute("messages.delete", peer_id=self.peer_id,
                               cmids=self.message_id, delete_for_all=1)

    async def pin(self) -> None:
        await self.bot.execute("messages.pin", peer_id=self.peer_id, conversation_message_id=self.message_id)


class MessageTemplate:
    def __init__(self, bot, text: str = None, attachment: str = None,
                 keyboard: Union[str, Keyboard] = None, forward: dict = None):
        self.bot = bot
        self.text = text
        self.attachment = attachment
        self.keyboard = keyboard

        self.forward = None
        if forward is not None:
            self.forward = json.dumps(forward)

    async def send_to(self, peer_id: Union[str, int]) -> Union[list[Message], Message]:
        response = await self.bot.execute("messages.send", message=self.text, peer_ids=peer_id,
                                          random_id=0, attachment=self.attachment, keyboard=self.keyboard,
                                          forward=self.forward)
        print(response)
        messages = []
        for message in response["response"]:
            messages.append(Message(self.bot, message["peer_id"], message["conversation_message_id"]))

        if len(messages) == 1:
            return messages[0]
        return messages
